package sun.misc;

import java.net.MalformedURLException;
import sun.net.www.ParseUtil;
import java.net.URL;
import sun.security.action.GetPropertyAction;
import java.security.PrivilegedAction;
import java.util.Enumeration;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.jar.Manifest;
import java.util.StringTokenizer;
import java.util.jar.Attributes;
import java.io.IOException;
import java.util.jar.JarFile;
import java.util.Vector;

public class ExtensionDependency
{
    private static Vector<ExtensionInstallationProvider> providers;
    static final boolean DEBUG = false;
    
    public ExtensionDependency() {
    }
    
    public static synchronized void addExtensionInstallationProvider(ExtensionInstallationProvider e) {
        if (ExtensionDependency.providers == null) {
            ExtensionDependency.providers = new Vector();
        }
        ExtensionDependency.providers.add(e);
    }
    
    public static synchronized void removeExtensionInstallationProvider(ExtensionInstallationProvider o) {
        ExtensionDependency.providers.remove(o);
    }
    
    public static boolean checkExtensionsDependencies(JarFile jarFile) {
        if (ExtensionDependency.providers == null) {
            return true;
        }
        try {
            return new ExtensionDependency().checkExtensions(jarFile);
        }
        catch (ExtensionInstallationException ex) {
            debug(ex.getMessage());
            return false;
        }
    }
    
    protected boolean checkExtensions(JarFile jarFile) throws ExtensionInstallationException {
        Manifest manifest;
        try {
            manifest = jarFile.getManifest();
        }
        catch (IOException ex) {
            return false;
        }
        if (manifest == null) {
            return true;
        }
        boolean n = true;
        Attributes mainAttributes = manifest.getMainAttributes();
        if (mainAttributes != null) {
            String value = mainAttributes.getValue(Attributes.Name.EXTENSION_LIST);
            if (value != null) {
                StringTokenizer stringTokenizer = new StringTokenizer(value);
                while (stringTokenizer.hasMoreTokens()) {
                    String nextToken = stringTokenizer.nextToken();
                    debug("The file " + jarFile.getName() + " appears to depend on " + nextToken);
                    String string = nextToken + "-" + Attributes.Name.EXTENSION_NAME.toString();
                    if (mainAttributes.getValue(string) == null) {
                        debug("The jar file " + jarFile.getName() + " appers to depend on " + nextToken + " but does not define the " + string + " attribute in its manifest ");
                    }
                    else if (!this.checkExtension(nextToken, mainAttributes)) {
                        debug("Failed installing " + nextToken);
                        n = false;
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                debug("No dependencies for " + jarFile.getName());
            }
        }
        return n;
    }
    
    protected synchronized boolean checkExtension(String str, Attributes attributes) throws ExtensionInstallationException {
        debug("Checking extension " + str);
        if (this.checkExtensionAgainstInstalled(str, attributes)) {
            return true;
        }
        debug("Extension not currently installed ");
        return this.installExtension(new ExtensionInfo(str, attributes), null);
    }
    
    boolean checkExtensionAgainstInstalled(String s, Attributes attributes) throws ExtensionInstallationException {
        File checkExtensionExists = this.checkExtensionExists(s);
        if (checkExtensionExists != null) {
            try {
                if (this.checkExtensionAgainst(s, attributes, checkExtensionExists)) {
                    return true;
                }
            }
            catch (FileNotFoundException ex4) {
                this.debugException(ex4);
            }
            catch (IOException ex5) {
                this.debugException(ex5);
            }
            return false;
        }
        File[] installedExtensions;
        try {
            installedExtensions = this.getInstalledExtensions();
        }
        catch (IOException ex) {
            this.debugException(ex);
            return false;
        }
        for (int i = 0; i < installedExtensions.length; ++i) {
            try {
                if (this.checkExtensionAgainst(s, attributes, installedExtensions[i])) {
                    return true;
                }
            }
            catch (FileNotFoundException ex2) {
                this.debugException(ex2);
            }
            catch (IOException ex3) {
                this.debugException(ex3);
            }
        }
        return false;
    }
    
    protected boolean checkExtensionAgainst(String str, Attributes attributes, final File file) throws IOException, FileNotFoundException, ExtensionInstallationException {
        debug("Checking extension " + str + " against " + file.getName());
        Manifest manifest;
        try {
            manifest = (Manifest)AccessController.doPrivileged(new PrivilegedExceptionAction<Manifest>() {
                @Override
                public Manifest run() throws IOException, FileNotFoundException {
                    if (!file.exists()) {
                        throw new FileNotFoundException(file.getName());
                    }
                    return new JarFile(file).getManifest();
                }
            });
        }
        catch (PrivilegedActionException ex) {
            if ((ex.getException() instanceof FileNotFoundException)) {
                throw (FileNotFoundException)ex.getException();
            }
            throw (IOException)ex.getException();
        }
        ExtensionInfo obj = new ExtensionInfo(str, attributes);
        debug("Requested Extension : " + obj);
        if (manifest != null) {
            Attributes mainAttributes = manifest.getMainAttributes();
            if (mainAttributes != null) {
                ExtensionInfo obj2 = new ExtensionInfo(null, mainAttributes);
                debug("Extension Installed " + obj2);
                switch (obj2.isCompatibleWith(obj)) {
                    case 0: {
                        debug("Extensions are compatible");
                        return true;
                    }
                    case 4: {
                        debug("Extensions are incompatible");
                        return false;
                    }
                    default: {
                        debug("Extensions require an upgrade or vendor switch");
                        return this.installExtension(obj, obj2);
                    }
                }
            }
        }
        return false;
    }
    
    protected boolean installExtension(ExtensionInfo extensionInfo, ExtensionInfo extensionInfo2) throws ExtensionInstallationException {
        Vector<Object> vector;
        synchronized (ExtensionDependency.providers) {
            vector = (vector = (Vector)ExtensionDependency.providers.clone());
        }
        Enumeration<Object> elements = vector.elements();
        while (elements.hasMoreElements()) {
            ExtensionInstallationProvider extensionInstallationProvider = (ExtensionInstallationProvider)elements.nextElement();
            if (extensionInstallationProvider != null && extensionInstallationProvider.installExtension(extensionInfo, extensionInfo2)) {
                debug(extensionInfo.name + " installation successful");
                this.addNewExtensionsToClassLoader((Launcher.ExtClassLoader)Launcher.getLauncher().getClassLoader().getParent());
                return true;
            }
            continue;
        }
        debug(extensionInfo.name + " installation failed");
        return false;
    }
    
    private File checkExtensionExists(final String s) {
        return (File)AccessController.doPrivileged(new PrivilegedAction<File>() {
            
            @Override
            public File run() {
                try {
                    File[] access$000 = ExtensionDependency.access$000();
                    for (int i = 0; i < access$000.length; ++i) {
                        int j = 0;
                        while (j < fileExt.length) {
                            Object o;
                            if (s.toLowerCase().endsWith(fileExt[j])) {
                                o = new File(access$000[i], s);
                            }
                            else {
                                o = new File(access$000[i], s + fileExt[j]);
                            }
                            ExtensionDependency.access$100("checkExtensionExists:fileName " + o.getName());
                            if (o.exists()) {
                                return o;
                            }
                            ++j;
                        }
                    }
                    return null;
                }
                catch (Exception ex) {
                    ExtensionDependency.access$200(ExtensionDependency.this, ex);
                    return null;
                }
            }
        });
    }
    
    private static File[] getExtDirs() {
        String str = (String)AccessController.doPrivileged(new GetPropertyAction("java.ext.dirs"));
        File[] obj;
        if (str != null) {
            StringTokenizer stringTokenizer = new StringTokenizer(str, File.pathSeparator);
            int countTokens = stringTokenizer.countTokens();
            debug("getExtDirs count " + countTokens);
            obj = new File[countTokens];
            for (int i = 0; i < countTokens; ++i) {
                obj[i] = new File(stringTokenizer.nextToken());
                debug("getExtDirs dirs[" + i + "] " + obj[i]);
            }
        }
        else {
            obj = new File[0];
            debug("getExtDirs dirs " + obj);
        }
        debug("getExtDirs dirs.length " + obj.length);
        return obj;
    }
    
    private static File[] getExtFiles(File[] array) throws IOException {
        Vector<Object> vector = new Vector();
        for (int i = 0; i < array.length; ++i) {
            String[] list = array[i].list(new JarFilter());
            if (list != null) {
                debug("getExtFiles files.length " + list.length);
                for (int j = 0; j < list.length; ++j) {
                    File o = new File(array[i], list[j]);
                    vector.add(o);
                    debug("getExtFiles f[" + j + "] " + o);
                }
            }
        }
        File[] anArray = new File[vector.size()];
        vector.copyInto(anArray);
        debug("getExtFiles ua.length " + anArray.length);
        return anArray;
    }
    
    private File[] getInstalledExtensions() throws IOException {
        return (File[])AccessController.doPrivileged(new PrivilegedAction<File[]>() {
            @Override
            public File[] run() {
                try {
                    return ExtensionDependency.access$300(ExtensionDependency.access$000());
                }
                catch (IOException ex) {
                    ExtensionDependency.access$100("Cannot get list of installed extensions");
                    ExtensionDependency.access$200(ExtensionDependency.this, ex);
                    return new File[0];
                }
            }
        });
    }
    
    private Boolean addNewExtensionsToClassLoader(Launcher.ExtClassLoader extClassLoader) {
        try {
            File[] installedExtensions = this.getInstalledExtensions();
            for (int i = 0; i < installedExtensions.length; ++i) {
                URL url = (URL)AccessController.doPrivileged(new PrivilegedAction<URL>() {
                    
                    @Override
                    public URL run() {
                        try {
                            return ParseUtil.fileToEncodedURL(instFile);
                        }
                        catch (MalformedURLException ex) {
                            ExtensionDependency.access$200(ExtensionDependency.this, ex);
                            return null;
                        }
                    }
                });
                if (url != null) {
                    URL[] urLs = extClassLoader.getURLs();
                    boolean n = false;
                    for (int j = 0; j < urLs.length; ++j) {
                        debug("URL[" + j + "] is " + urLs[j] + " looking for " + url);
                        if (urLs[j].toString().compareToIgnoreCase(url.toString()) == 0) {
                            n = true;
                            debug("Found !");
                        }
                    }
                    if (n == 0) {
                        debug("Not Found ! adding to the classloader " + url);
                        extClassLoader.addExtURL(url);
                    }
                }
            }
        }
        catch (MalformedURLException ex) {
            ex.printStackTrace();
        }
        catch (IOException ex2) {
            ex2.printStackTrace();
        }
        return Boolean.TRUE;
    }
    
    private static void debug(String s) {
    }
    
    private void debugException(Throwable t) {
    }
    
    static File[] access$000() {
        return getExtDirs();
    }
    
    static void access$100(String s) {
        debug(s);
    }
    
    static void access$200(ExtensionDependency extensionDependency, Throwable t) {
        extensionDependency.debugException(t);
    }
    
    static File[] access$300(File[] array) throws IOException {
        return getExtFiles(array);
    }
}
